﻿local buttons = {
	MultiBarRightButton1,
	MultiBarRightButton2,
	MultiBarRightButton3,
	MultiBarRightButton4,
	MultiBarRightButton5,
	MultiBarRightButton6,
	MultiBarRightButton7,
	MultiBarRightButton8,
	MultiBarRightButton9,
	MultiBarRightButton10,
	MultiBarRightButton11,
	MultiBarRightButton12,
	MultiBarLeftButton1,
	MultiBarLeftButton2,
	MultiBarLeftButton3,
	MultiBarLeftButton4,
	MultiBarLeftButton5,
	MultiBarLeftButton6,
	MultiBarLeftButton7,
	MultiBarLeftButton8,
	MultiBarLeftButton8,
	MultiBarLeftButton9,
	MultiBarLeftButton10,
	MultiBarLeftButton11,
	MultiBarLeftButton12,
}

local addon = CreateFrame("Frame", "AutoHideRightBar", UIParent);
local lock = CreateFrame("button", "AutoHideRightBarLock", addon);
addon:SetFrameLevel(1);
addon:RegisterEvent("PLAYER_ENTERING_WORLD");
local onload = false;
local locked = false;
addon:SetScript("OnEvent", function(_, event, ...)
	if event == "PLAYER_ENTERING_WORLD" and not onload then
		onload = true;
		lock:SetSize(64, 32);
		lock:ClearAllPoints();
		lock:Show();

		local tex = lock:CreateTexture("normal", "BACKGROUND");
		tex:SetAllPoints(lock);
		tex:SetTexture("Interface/Buttons/UI-Button-KeyRing");
		tex:SetTexCoord(1,0,0,0,1,1,0,1);
	
		local tex2 = lock:CreateTexture("down", "ARTWORK");
		tex2:SetAllPoints(lock);
		tex2:SetTexture("Interface/Buttons/UI-Button-KeyRing-Down");
		tex2:SetTexCoord(1,0,0,0,1,1,0,1);
		tex2:Hide();

		local tex3 = lock:CreateTexture("highlight", "OVERLAY");
		tex3:SetAllPoints(lock);
		tex3:SetTexture("Interface/Buttons/UI-Button-KeyRing-Highlight");
		tex3:SetTexCoord(1,0,0,0,1,1,0,1);
		tex3:SetBlendMode("ADD");
		tex3:Hide();

		lock.normal = tex;
		lock.down = tex2;
		lock.highlight = tex3;
	end
end);

local function SetTracker()
	if not ObjectiveTrackerFrame then
		return;
	elseif not ObjectiveTrackerFrame:IsShown() then
		return;
	end
	
	local a1, f1, b1, x1, y1 = ObjectiveTrackerFrame:GetPoint(1);
	local a2, f2, b2, x2, y2 = ObjectiveTrackerFrame:GetPoint(2);
	if f1 ~= MinimapCluster or f2 ~= UIParent then
		return;
	elseif a1 ~= "TOPRIGHT" or b1 ~= "BOTTOMRIGHT" then
		return;
	elseif a2 ~= "BOTTOMRIGHT" or b2 ~= "BOTTOMRIGHT" then
		return;
	end

	local x;
	if MultiBarRight:GetAlpha() > 0 then
		x = -12 - VerticalMultiBarsContainer:GetSize();
	else
		x = -12;
	end

	ObjectiveTrackerFrame:SetPoint("TOPRIGHT", MinimapCluster, "BOTTOMRIGHT", x, y1);
	ObjectiveTrackerFrame:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMRIGHT", x, y2);
end

local frames = {
	AutoHideRightBar,
	MultiBarLeft,
	MultiBarRight,
	AutoHideRightBarLock,
}

local function HideRightBar()
	if locked then
		return
	end
	if not addon:CheckMouse() then
		MultiBarRight:SetAlpha(0);
		MultiBarLeft:SetAlpha(0);
		for id, button in pairs(buttons) do
			button.cooldown:Hide();
		end
		SetTracker();
		AutoHideRightBarLock:Hide();
	end
end

function addon:CheckMouse()
	local focus = GetMouseFocus();
	if focus and focus ~= WorldFrame and (focus:GetName() == "SpellFlyout" or focus:GetParent():GetName() == "SpellFlyout") then
		if not focus.hooked then
			focus:HookScript("OnLeave", function()
				HideRightBar();
			end);
			focus.hooked = true;
		end
		return true;
	end
	return tContains(frames, GetMouseFocus()) or tContains(buttons, GetMouseFocus())
end

local function ShowRightBar()
	if SHOW_MULTI_ACTIONBAR_3 then
		MultiBarRight:SetAlpha(1);
		for id, button in pairs(buttons) do
			if id < 13 then
				if button.icon:IsShown() then
					button.cooldown:Show();
				end
			end
		end

		if SHOW_MULTI_ACTIONBAR_4 then
			MultiBarLeft:SetAlpha(1);
			for id, button in pairs(buttons) do
				if id > 12 then
					if button.icon:IsShown() then
						button.cooldown:Show();
					end
				end
			end
		end
		SetTracker();
		AutoHideRightBarLock:Show();
	end
end

addon:SetScript("OnEnter", function(self)
	if MultiBarRight:GetAlpha() == 0 then
		ShowRightBar();
	end
end);

addon:SetScript("OnLeave", function(self)
	if MultiBarRight:GetAlpha() > 0 then
		HideRightBar();
	end
end);

AutoHideRightBarLock:SetScript("OnEnter", function(self)
	self.highlight:Show();
end);

AutoHideRightBarLock:SetScript("OnLeave", function(self)
	if MultiBarRight:GetAlpha() > 0 then
		HideRightBar();
	end
	self.highlight:Hide();
	if not locked then
		self.down:Hide();
	end
end);

AutoHideRightBarLock:SetScript("OnMouseDown", function(self)
	self.down:Show();
end);

AutoHideRightBarLock:SetScript("OnMouseUp", function(self)
	self.down:Hide();
end);

AutoHideRightBarLock:SetScript("Onclick", function(self)
	locked = not locked;
	if locked then
		self.down:Show();
	end
end);

if ObjectiveTrackerFrame and ObjectiveTrackerFrame.HeaderMenu then
	local w, h = ObjectiveTrackerFrame:GetSize();
	ObjectiveTrackerFrame.HeaderMenu:SetSize(w,30);
	ObjectiveTrackerFrame.HeaderMenu:SetScript("OnMouseUp", function(self)
		ObjectiveTracker_MinimizeButton_OnClick();
	end);
end

for i, button in pairs(buttons) do
	button:HookScript("OnEnter", function(self)
		ShowRightBar();
	end);
	button:HookScript("OnLeave", function(self)
		HideRightBar();
	end);
	button.cooldown:HookScript("OnShow", function(self)
		if MultiBarRight:GetAlpha() == 0 then
			self:Hide();
		end
	end);
end

hooksecurefunc(VerticalMultiBarsContainer, "SetSize", function(self, ...)
	local w, h = ...;
	addon:SetSize(w, h + 32);
	addon:ClearAllPoints();
	addon:SetPoint(VerticalMultiBarsContainer:GetPoint());
	HideRightBar();
end);

hooksecurefunc("QuestObjectiveSetupBlockButton_AddRightButton", function(block, button, initialAnchorOffsets)
	local x;
	if block.HeaderText then
		x = -54
	else
		x = -36
	end
	button:ClearAllPoints();
	if block.groupFinderButton and block.groupFinderButton ~= block.rightButton then
		button:SetPoint("RIGHT", block.rightButton, "LEFT", 4, 3);
	else
		button:SetPoint("TOPLEFT", block, "TOPLEFT", x, 3);
	end
end)

hooksecurefunc("UIParent_ManageFramePositions", function() 
	SetTracker();
	if SHOW_MULTI_ACTIONBAR_4 then
		lock:SetPoint("BOTTOMLEFT", MultiBarRight, "TOPLEFT", -20, 0);
	else
		lock:SetPoint("BOTTOMLEFT", MultiBarRight, "TOPLEFT", 1, 0);
	end
end);